@@ -54,6 +54,8 @@ function InsertBlockItem(guiAction) {
     if (!isNull(guiAction)) {
         this.url = this.guiAction.data();
     }
+
+    this.autoTerminate = false;
 }
 
 InsertBlockItem.State = {
@@ -64,42 +66,51 @@ InsertBlockItem.prototype = new BlockInsert();
 
 InsertBlockItem.prototype.beginEvent = function() {
     // part library item is loaded into this document:
-    var ms = new RMemoryStorage();
-    var si = new RSpatialIndexNavel();
-    this.docItem = new RDocument(ms, si);
-    this.diItem = new RDocumentInterface(this.docItem);
-    this.diItem.setNotifyListeners(false);
+    if (isNull(this.diItem)) {
+        var ms = new RMemoryStorage();
+        var si = new RSpatialIndexNavel();
+        this.docItem = new RDocument(ms, si);
+        this.diItem = new RDocumentInterface(this.docItem);
+        this.diItem.setNotifyListeners(false);
+    }
 
     // TODO refactor
     BlockInsert.prototype.beginEvent.call(this);
 
-    var path;
-    var err;
-    if (this.url.isLocalFile()) {
-        qDebug("importing file: " + this.url);
-        path = this.url.toLocalFile();
-        err = this.diItem.importFile(path, "", false);
-    }
-    else {
-        qDebug("importing URL: " + this.url);
-        if (isFunction(this.url.encodedPath)) {
-            path = QUrl.fromPercentEncoding(this.url.encodedPath());
+    var path = undefined;
+    if (!isNull(this.url)) {
+        var err;
+        if (this.url.isLocalFile()) {
+            path = this.url.toLocalFile();
+            qDebug("importing file: " + path);
+            err = this.diItem.importFile(path, "", false);
         }
         else {
-            path = this.url.path();
+            qDebug("importing URL: " + this.url);
+            if (isFunction(this.url.encodedPath)) {
+                path = QUrl.fromPercentEncoding(this.url.encodedPath());
+            }
+            else {
+                path = this.url.path();
+            }
+
+            err = this.diItem.importUrl(this.url, "", false);
         }
 
-        err = this.diItem.importUrl(this.url, "", false);
-    }
-
-    if (err!==RDocumentInterface.IoErrorNoError) {
-        EAction.handleUserWarning(qsTr("Cannot import file from URL: ") + this.url.toString());
-        this.terminate();
+        if (err!==RDocumentInterface.IoErrorNoError) {
+            EAction.handleUserWarning(qsTr("Cannot import file from URL: ") + this.url.toString());
+            this.terminate();
+            return;
+        }
     }
 
     // no block name given:
     // create block name from file name:
     if (isNull(this.blockName)) {
+        if (isNull(path)) {
+            this.terminate();
+            return;
+        }
         this.blockName = new QFileInfo(path).completeBaseName();
 
         // fix block name if necessary:
@@ -117,7 +128,6 @@ InsertBlockItem.prototype.beginEvent = function() {
     }
 
     if (this.docItem.hasBlock(this.blockName)) {
-        // 20140520:
         // if the item contains a block with the same name as the file base name,
         // insert without creating a new block:
         this.blockName = undefined;
@@ -204,12 +214,18 @@ InsertBlockItem.prototype.pickCoordinate = function(event, preview) {
             this.updatePreview();
         }
         else {
+            var prev = true;
             var op = this.getOperation();
             if (!isNull(op)) {
-                di.applyOperation(op);
+                var t = di.applyOperation(op);
+                prev = t.isPreview();
                 di.clearPreview();
                 di.repaintViews();
             }
+
+            if (this.autoTerminate===true && !prev) {
+                this.terminate();
+            }
         }
     }
 };
