@@ -107,6 +107,7 @@ Print.prototype.print = function(pdfFile) {
             Print.printDialog.rejected.connect(function() { Print.cancel = true; });
             Print.printDialog.exec();
             Print.printDialog.destroy();
+            EAction.activateMainWindow();
         }
         else {
             // Mac, Win: exec() never returns without destroying the dialog through these signals:
@@ -117,6 +118,7 @@ Print.prototype.print = function(pdfFile) {
                             Print.printDialog.close();
                             if (RS.getSystemId()==="osx") {
                                 Print.printDialog.destroy();
+                                EAction.activateMainWindow();
                             }
                         });
 
@@ -126,6 +128,7 @@ Print.prototype.print = function(pdfFile) {
                             Print.printDialog.close();
                             if (RS.getSystemId()==="osx") {
                                 Print.printDialog.destroy();
+                                EAction.activateMainWindow();
                             }
                         });
 
@@ -271,7 +274,7 @@ Print.prototype.print = function(pdfFile) {
             );
         }
 
-        if (Print.getEnablePageTags()) {
+        if (Print.getEnablePageTags(this.document)) {
             Print.drawPageTags(this.document, painter, i, pageBorderTransformed, true, printerFactor);
         }
     }
@@ -298,10 +301,6 @@ Print.prototype.print = function(pdfFile) {
  * \param rect QRectF object that represents the page border.
  */
 Print.prototype.printPage = function(painter, rect) {
-    //var document = EAction.getDocument();
-    //var mdiChild = EAction.getMdiChild();
-    //var view = mdiChild.getLastKnownViewWithFocus();
-
     this.view.paintEntities(
         painter,
         new RBox(
@@ -524,12 +523,12 @@ Print.drawPageTags = function(document, painter, index, border, printing, printe
     correctedBorder.setTop(correctedBorder.bottom());
     correctedBorder.setBottom(top);
 
-    font = Print.getTagFont();
+    font = Print.getTagFont(document);
     font.setPointSizeF(1.0);
 
     var sf = 1.0;
     // font size is given in points:
-    sf *= Print.getTagFont().pointSizeF();
+    sf *= Print.getTagFont(document).pointSizeF();
     // convert points to mm:
     sf = sf * 25.4 / 72;
 
@@ -554,12 +553,12 @@ Print.drawPageTags = function(document, painter, index, border, printing, printe
     t.scale(sf,-sf);
     path = t.map(path);
 
-    var outside = (Print.getTagAlignment() === "Outside");
+    var outside = (Print.getTagAlignment(document) === "Outside");
 
     width = path.boundingRect().width();
     height = path.boundingRect().height();
 
-    switch (Print.getTagPosition()) {
+    switch (Print.getTagPosition(document)) {
     case "Top":
     case "Bottom":
         x = (correctedBorder.left() + correctedBorder.right()) / 2 - width / 2;
@@ -583,7 +582,7 @@ Print.drawPageTags = function(document, painter, index, border, printing, printe
         break;
     }
 
-    switch (Print.getTagPosition()) {
+    switch (Print.getTagPosition(document)) {
     case "Top":
     case "TopLeft":
     case "TopRight":
@@ -742,13 +741,13 @@ Print.getTransformed = function(document, rect) {
 };
 
 Print.getColorMode = function(document) {
-    var colorModeString = EAction.getValue("ColorSettings/ColorMode", "FullColor", document);
+    var colorModeString = Print.getValue("ColorSettings/ColorMode", "FullColor", document);
     return Print.getColorModeEnum(colorModeString);
 };
 
 Print.setColorMode = function(document, colorMode) {
     if (isString(colorMode)) {
-        document.setVariable("ColorSettings/ColorMode", colorMode);
+        Print.setValue("ColorSettings/ColorMode", colorMode, document);
     }
     else {
         Print.setColorMode(document, Print.getColorModeString(colorMode));
@@ -756,11 +755,11 @@ Print.setColorMode = function(document, colorMode) {
 };
 
 Print.getHairlineMode = function(document) {
-    return EAction.getBoolValue("Print/HairlineMode", false, document);
+    return Print.getBoolValue("Print/HairlineMode", false, document);
 };
 
 Print.setHairlineMode = function(document, onOff) {
-    document.setVariable("Print/HairlineMode", onOff);
+    Print.setValue("Print/HairlineMode", onOff, document);
 };
 
 /**
@@ -841,8 +840,81 @@ Print.getDefaultPaperRect = function(document, paperUnit) {
     return new QRectF(x, y, w, h);
 };
 
+/**
+ * \return Value for given variable from current layout block, document or settings.
+ */
+Print.getValue = function(key, def, document) {
+    var ret = def;
+
+    if (isNull(document) || document.getCurrentBlockId()===document.getModelSpaceBlockId()) {
+        ret = EAction.getValue(key, def, document);
+    }
+    else {
+        var block = document.queryCurrentBlock();
+        if (!isNull(block)) {
+            // fall back on document wide value for blocks:
+            def = EAction.getValue(key, def, document);
+
+            ret = block.getCustomProperty("QCAD", key, def);
+        }
+    }
+
+    return ret;
+};
+
+Print.getDoubleValue = function(key, def, document) {
+    var val = Print.getValue(key, def, document);
+    val = parseFloat(val);
+    if (!RMath.isSane(val)) {
+        val = def;
+    }
+    return val;
+};
+
+Print.getIntValue = function(key, def, document) {
+    var val = Print.getValue(key, def, document);
+    val = parseInt(val);
+    if (!RMath.isSane(val)) {
+        val = def;
+    }
+    return val;
+};
+
+Print.getBoolValue = function(key, def, document) {
+    var val = Print.getValue(key, def, document);
+    if (typeof(val)==="string") {
+        return val==="true" || val==="yes";
+    }
+    if (typeof(val)==="number") {
+        return parseInt(val)===1
+    }
+
+    return val;
+};
+
+Print.setValue = function(key, val, document) {
+    if (isNull(document) || document.getCurrentBlockId()===document.getModelSpaceBlockId()) {
+        document.setVariable(key, val);
+    }
+    else {
+        var block = document.queryCurrentBlock();
+        //if (block.hasLayout()) {
+        if (block.getCustomProperty("QCAD", key, undefined)!==val) {
+            block.setCustomProperty("QCAD", key, val.toString());
+            var di = EAction.getDocumentInterface();
+            if (!isNull(di)) {
+                var op = new RAddObjectOperation(block);
+                op.setRecordAffectedObjects(false);
+                op.setSpatialIndexDisabled(true);
+                di.applyOperation(op);
+            }
+        }
+        //}
+    }
+};
+
 Print.getGlueMarginLeft = function(document) {
-    return EAction.getDoubleValue("MultiPageSettings/GlueMarginsLeft", Print.getDefaultPrintMarginLeft(document), document);
+    return Print.getDoubleValue("MultiPageSettings/GlueMarginsLeft", Print.getDefaultPrintMarginLeft(document), document);
 };
 
 Print.getDefaultPrintMarginLeft = function(document, paperUnit) {
@@ -851,7 +923,7 @@ Print.getDefaultPrintMarginLeft = function(document, paperUnit) {
 };
 
 Print.getGlueMarginTop = function(document) {
-    return EAction.getDoubleValue( "MultiPageSettings/GlueMarginsTop", Print.getDefaultPrintMarginTop(document), document);
+    return Print.getDoubleValue("MultiPageSettings/GlueMarginsTop", Print.getDefaultPrintMarginTop(document), document);
 };
 
 Print.getDefaultPrintMarginTop = function(document, paperUnit) {
@@ -860,7 +932,7 @@ Print.getDefaultPrintMarginTop = function(document, paperUnit) {
 };
 
 Print.getGlueMarginRight = function(document) {
-    return EAction.getDoubleValue( "MultiPageSettings/GlueMarginsRight", Print.getDefaultPrintMarginRight(document), document);
+    return Print.getDoubleValue("MultiPageSettings/GlueMarginsRight", Print.getDefaultPrintMarginRight(document), document);
 };
 
 Print.getDefaultPrintMarginRight = function(document, paperUnit) {
@@ -870,7 +942,7 @@ Print.getDefaultPrintMarginRight = function(document, paperUnit) {
 };
 
 Print.getGlueMarginBottom = function(document) {
-    return EAction.getDoubleValue( "MultiPageSettings/GlueMarginsBottom", Print.getDefaultPrintMarginBottom(document), document);
+    return Print.getDoubleValue("MultiPageSettings/GlueMarginsBottom", Print.getDefaultPrintMarginBottom(document), document);
 };
 
 Print.getDefaultPrintMarginBottom = function(document, paperUnit) {
@@ -880,16 +952,16 @@ Print.getDefaultPrintMarginBottom = function(document, paperUnit) {
 };
 
 Print.getOffset = function(document) {
-    var x = EAction.getDoubleValue("PageSettings/OffsetX", 0, document);
-    var y = EAction.getDoubleValue("PageSettings/OffsetY", 0, document);
-//    x += Print.getGlueMarginLeft();
-//    y += Print.getGlueMarginBottom();
+    var x = Print.getDoubleValue("PageSettings/OffsetX", 0, document);
+    var y = Print.getDoubleValue("PageSettings/OffsetY", 0, document);
+//    x += Print.getGlueMarginLeft(document);
+//    y += Print.getGlueMarginBottom(document);
     return new RVector(x, y);
 };
 
 Print.setOffset = function(document, offset) {
-    document.setVariable("PageSettings/OffsetX", offset.x);
-    document.setVariable("PageSettings/OffsetY", offset.y);
+    Print.setValue("PageSettings/OffsetX", offset.x, document);
+    Print.setValue("PageSettings/OffsetY", offset.y, document);
 };
 
 /**
@@ -929,11 +1001,11 @@ Print.getDefaultPaperSizeMM = function() {
  * \return Paper width in PaperUnit.
  */
 Print.getPaperWidth = function(document) {
-    var defaultPaperSizeMM = Print.defaultPrinter.paperSize(QPrinter.Millimeter);
+    var defaultPaperSizeMM = Print.getDefaultPaperSizeMM();
     var dwMM = defaultPaperSizeMM.width();
     var dw = RUnit.convert(dwMM, RS.Millimeter, Print.getPaperUnit(document));
 
-    var w = EAction.getValue("PageSettings/PaperWidth", dw, document);
+    var w = Print.getValue("PageSettings/PaperWidth", dw, document);
 
     // compat:
     if (isArray(w)) {
@@ -951,11 +1023,11 @@ Print.getPaperWidth = function(document) {
  * \return Paper height in PaperUnit.
  */
 Print.getPaperHeight = function(document) {
-    var defaultPaperSizeMM = Print.defaultPrinter.paperSize(QPrinter.Millimeter);
+    var defaultPaperSizeMM = Print.getDefaultPaperSizeMM();
     var dhMM = defaultPaperSizeMM.height();
     var dh = RUnit.convert(dhMM, RS.Millimeter, Print.getPaperUnit(document));
 
-    var h = EAction.getValue("PageSettings/PaperHeight", dh, document);
+    var h = Print.getValue("PageSettings/PaperHeight", dh, document);
     // compat:
     if (isArray(h)) {
         h = h[0];
@@ -972,7 +1044,8 @@ Print.getPaperHeight = function(document) {
  * \return Paper unit enum.
  */
 Print.getPaperUnit = function(document) {
-    return EAction.getIntValue("UnitSettings/PaperUnit", RS.Millimeter, document);
+    var def = Print.getDefaultPaperUnit();
+    return Print.getIntValue("UnitSettings/PaperUnit", def, document);
 };
 
 Print.getDefaultPaperUnit = function() {
@@ -1117,15 +1190,15 @@ Print.getPageOrientationEnum = function(document) {
 };
 
 Print.getPageOrientationString = function(document) {
-    return EAction.getValue("PageSettings/PageOrientation", "Portrait", document);
+    return Print.getValue("PageSettings/PageOrientation", "Portrait", document);
 };
 
 Print.setPageOrientationString = function(document, pageOrientation) {
-    document.setVariable("PageSettings/PageOrientation", pageOrientation);
+    Print.setValue("PageSettings/PageOrientation", pageOrientation, document);
 };
 
 Print.setPageOrientationEnum = function(document, pageOrientation) {
-    if (pageOrientation==QPrinter.Landscape) {
+    if (pageOrientation.valueOf()===QPrinter.Landscape.valueOf()) {
         Print.setPageOrientationString(document, "Landscape");
     }
     else {
@@ -1134,23 +1207,23 @@ Print.setPageOrientationEnum = function(document, pageOrientation) {
 };
 
 Print.getShowPaperBorders = function(document) {
-    return EAction.getBoolValue("PageSettings/ShowPaperBorders", true, document);
+    return Print.getBoolValue("PageSettings/ShowPaperBorders", true, document);
 };
 
 Print.setShowPaperBorders = function(document, showPaperBorders) {
-    document.setVariable("PageSettings/ShowPaperBorders", showPaperBorders);
+    Print.setValue("PageSettings/ShowPaperBorders", showPaperBorders, document);
 };
 
 Print.getPrintCropMarks = function(document) {
-    return EAction.getBoolValue("MultiPageSettings/PrintCropMarks", true, document);
+    return Print.getBoolValue("MultiPageSettings/PrintCropMarks", true, document);
 };
 
 Print.setPrintCropMarks = function(document, printCropMarks) {
-    document.setVariable("MultiPageSettings/PrintCropMarks", printCropMarks);
+    Print.setValue("MultiPageSettings/PrintCropMarks", printCropMarks, document);
 };
 
 Print.getScaleString = function(document) {
-    var str = EAction.getStringValue("PageSettings/Scale", "1:1", document);
+    var str = Print.getValue("PageSettings/Scale", "1:1", document);
 
     // round:
     if (/^[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?$/.test(str)) {
@@ -1204,7 +1277,7 @@ Print.setScale = function(document, scale) {
 };
 
 Print.setScaleString = function(document, scaleString) {
-    document.setVariable("PageSettings/Scale", scaleString);
+    Print.setValue("PageSettings/Scale", scaleString, document);
 };
 
 Print.getColumns = function(document) {
@@ -1213,11 +1286,11 @@ Print.getColumns = function(document) {
         return 1;
     }
 
-    return EAction.getIntValue("MultiPageSettings/Columns", 1, document);
+    return Print.getIntValue("MultiPageSettings/Columns", 1, document);
 };
 
 Print.setColumns = function(document, columns) {
-    document.setVariable("MultiPageSettings/Columns", columns);
+    Print.setValue("MultiPageSettings/Columns", columns, document);
 };
 
 Print.getRows = function(document) {
@@ -1226,16 +1299,16 @@ Print.getRows = function(document) {
         return 1;
     }
 
-    return EAction.getIntValue("MultiPageSettings/Rows", 1, document);
+    return Print.getIntValue("MultiPageSettings/Rows", 1, document);
 };
 
 Print.setRows = function(document, rows) {
-    document.setVariable("MultiPageSettings/Rows", rows);
+    Print.setValue("MultiPageSettings/Rows", rows, document);
 };
 
 Print.getBackgroundColor = function(document) {
     // this might return a string or an RColor object:
-    var color = EAction.getValue("ColorSettings/BackgroundColor", new RColor("white"), document);
+    var color = Print.getValue("ColorSettings/BackgroundColor", new RColor("white"), document);
 
     if (isOfType(color, RColor)) {
         return color;
@@ -1255,15 +1328,15 @@ Print.getBackgroundColor = function(document) {
 };
 
 Print.getEnablePageTags = function(document) {
-    return EAction.getBoolValue("PageTagSettings/EnablePageTags", false, document);
+    return Print.getBoolValue("PageTagSettings/EnablePageTags", false, document);
 };
 
-Print.getIdentifyPageTags = function(document) {
-    return EAction.getBoolValue("PageTagSettings/IdentifyPageTags", false, document);
-};
+//Print.getIdentifyPageTags = function(document) {
+//    return Print.getBoolValue("PageTagSettings/IdentifyPageTags", false, document);
+//};
 
 Print.getTagFont = function(document) {
-    var ret = EAction.getValue("PageTagSettings/TagFont", new QFont(), document);
+    var ret = Print.getValue("PageTagSettings/TagFont", new QFont(), document);
     if (isOfType(ret, QFont)) {
         return ret;
     }
@@ -1279,16 +1352,19 @@ Print.getTagFont = function(document) {
 };
 
 Print.getTagPosition = function(document) {
-    return EAction.getValue("PageTagSettings/TagPosition", "TopLeft", document);
+    return Print.getValue("PageTagSettings/TagPosition", "TopLeft", document);
 };
 
 Print.getTagAlignment = function(document) {
-    return EAction.getValue("PageTagSettings/TagAlignment", "Inside", document);
+    return Print.getValue("PageTagSettings/TagAlignment", "Inside", document);
 };
 
 /**
  * Parses the given scale string (e.g. "1:2") and returns the scale as number (e.g. 0.5).
  */
 Print.parseScale = function(scaleString) {
+    if (!isString(scaleString)) {
+        return 1.0;
+    }
     return RMath.parseScale(scaleString);
 };
